import axios from "axios"
import store from '@/store';

export const RESET = 'RESET';


const initialState = () => ({
  form: {
    colors_style: {
      value: null,
      options: [],
      fields: [],
    },
    color: {
      id_for_label: "couleur",
      label: "Couleur",
      field: {
        max_length: 128, // ! Vérifier la valeur dans django
      },
      html_name: "couleur",
      value: "#000000",
    },
    title: {
      errors: [],
      id_for_label: "title",
      label: "Titre",
      field: {
        max_length: 128, // ! Vérifier la valeur dans django
      },
      html_name: "title",
      value: null,
    },
    geom_type: {
      id_for_label: "geom_type",
      label: "Type de géométrie",
      field: {
        choices: ["Ligne", "Point", "Polygone"],
        max_length: 128, // ! Vérifier la valeur dans django
      },
      html_name: "geom_type",
      value: "Point",
    },
  },
  colorsStyleList: [],
  customForms: [],
  current_feature_type_slug: null,
  feature_types: [],
});

const feature_type = {
  namespaced: true,
  state: initialState(),
  
  mutations: {
    [RESET]: (state) => {
      const newState = initialState();
      Object.keys(newState).forEach((key) => {
        state[key] = newState[key];
      });
    },
    SET_FEATURE_TYPES(state, feature_types) {
      state.feature_types = feature_types;
    },
    SET_CURRENT_FEATURE_TYPE_SLUG(state, payload) {
      state.current_feature_type_slug = payload;
    },
    UPDATE_FORM(state, payload) {
      state.form = payload;
    },
    ADD_CUSTOM_FORM(state, customForm) {
      state.customForms = [...state.customForms, customForm ];
    },
    UPDATE_CUSTOM_FORM(state, payload) {
      const index = state.customForms.findIndex((el) => el.dataKey === payload.dataKey);
      if (index !== -1) state.customForms[index] = payload
    },
    REMOVE_CUSTOM_FORM(state, payload) {
      state.customForms = state.customForms.filter(form => form.dataKey !== payload);
    },
    UPDATE_COLOR_STYLE(state) { // * un peu dirty (mais pas quick) car getter pas réactif avec nested et compliqué de faire dans composants enfants multiples
      let res = [];
      for (let form of state.customForms) {
        if (form.label) res.push(form);
      }
      state.colorsStyleList = res;
    }
  },
  getters: {
    feature_type: state => state.feature_types.find(
      (el) => el.slug === state.current_feature_type_slug
    )
  },
  actions: {
    [RESET]: ({ commit }) => {
      commit("RESET_STATE");
    },
    POST_FEATURE_TYPE({ state }) {
      const data = { form: state.form, formset: state.customForms }
      console.log("data", data)

      axios
        .post(`${process.env.VUE_APP_DJANGO_API_BASE}feature_type/`, data)
        .then((response) => {
          const routerHistory = this.$router.options.routerHistory
          store.commit("SET_USER", response.data.user);
          this.$router.push(routerHistory[routerHistory.length - 1] || "/")
          store.dispatch("GET_USER_LEVEL_PROJECTS");
        })
        .catch(() => {
          store.commit("SET_USER", false)
        });
    },
  },

}

export default feature_type